package javax.management.remote.rmi;

import sun.reflect.misc.ReflectUtil;
import java.util.Arrays;
import javax.management.RuntimeOperationsException;
import com.sun.jmx.remote.util.OrderClassLoaders;
import java.rmi.UnmarshalException;
import javax.management.remote.JMXServerErrorException;
import java.security.PrivilegedExceptionAction;
import javax.management.remote.NotificationResult;
import javax.management.ListenerNotFoundException;
import javax.management.NotificationFilter;
import javax.management.IntrospectionException;
import javax.management.MBeanInfo;
import javax.management.InvalidAttributeValueException;
import javax.management.Attribute;
import javax.management.AttributeList;
import javax.management.AttributeNotFoundException;
import com.sun.jmx.mbeanserver.Util;
import javax.management.QueryExp;
import java.util.Set;
import java.rmi.MarshalledObject;
import javax.management.InstanceNotFoundException;
import java.security.PrivilegedActionException;
import javax.management.NotCompliantMBeanException;
import javax.management.MBeanException;
import javax.management.MBeanRegistrationException;
import javax.management.InstanceAlreadyExistsException;
import javax.management.ReflectionException;
import javax.management.ObjectInstance;
import javax.management.ObjectName;
import java.io.IOException;
import java.security.ProtectionDomain;
import java.security.Permissions;
import com.sun.jmx.remote.util.EnvHelp;
import java.security.AccessController;
import javax.management.MBeanPermission;
import java.security.Permission;
import java.security.PrivilegedAction;
import javax.management.loading.ClassLoaderRepository;
import com.sun.jmx.remote.security.JMXSubjectDomainCombiner;
import java.util.Collections;
import com.sun.jmx.remote.util.ClassLogger;
import java.util.Map;
import com.sun.jmx.remote.internal.ServerNotifForwarder;
import com.sun.jmx.remote.internal.ServerCommunicatorAdmin;
import com.sun.jmx.remote.util.ClassLoaderWithRepository;
import javax.management.MBeanServer;
import java.security.AccessControlContext;
import com.sun.jmx.remote.security.SubjectDelegator;
import javax.security.auth.Subject;
import java.rmi.server.Unreferenced;

public class RMIConnectionImpl implements RMIConnection, Unreferenced
{
    private static final Object[] NO_OBJECTS;
    private static final String[] NO_STRINGS;
    private final Subject subject;
    private final SubjectDelegator subjectDelegator;
    private final boolean removeCallerContext;
    private final AccessControlContext acc;
    private final RMIServerImpl rmiServer;
    private final MBeanServer mbeanServer;
    private final ClassLoader defaultClassLoader;
    private final ClassLoader defaultContextClassLoader;
    private final ClassLoaderWithRepository classLoaderWithRepository;
    private boolean terminated;
    private final String connectionId;
    private final ServerCommunicatorAdmin serverCommunicatorAdmin;
    private static final int ADD_NOTIFICATION_LISTENERS = 1;
    private static final int ADD_NOTIFICATION_LISTENER_OBJECTNAME = 2;
    private static final int CREATE_MBEAN = 3;
    private static final int CREATE_MBEAN_PARAMS = 4;
    private static final int CREATE_MBEAN_LOADER = 5;
    private static final int CREATE_MBEAN_LOADER_PARAMS = 6;
    private static final int GET_ATTRIBUTE = 7;
    private static final int GET_ATTRIBUTES = 8;
    private static final int GET_DEFAULT_DOMAIN = 9;
    private static final int GET_DOMAINS = 10;
    private static final int GET_MBEAN_COUNT = 11;
    private static final int GET_MBEAN_INFO = 12;
    private static final int GET_OBJECT_INSTANCE = 13;
    private static final int INVOKE = 14;
    private static final int IS_INSTANCE_OF = 15;
    private static final int IS_REGISTERED = 16;
    private static final int QUERY_MBEANS = 17;
    private static final int QUERY_NAMES = 18;
    private static final int REMOVE_NOTIFICATION_LISTENER = 19;
    private static final int REMOVE_NOTIFICATION_LISTENER_OBJECTNAME = 20;
    private static final int REMOVE_NOTIFICATION_LISTENER_OBJECTNAME_FILTER_HANDBACK = 21;
    private static final int SET_ATTRIBUTE = 22;
    private static final int SET_ATTRIBUTES = 23;
    private static final int UNREGISTER_MBEAN = 24;
    private ServerNotifForwarder serverNotifForwarder;
    private Map<String, ?> env;
    private static final ClassLogger logger;
    
    public RMIConnectionImpl(RMIServerImpl rmiServer, String connectionId, final ClassLoader defaultClassLoader, Subject subject, Map<String, ?> emptyMap) {
        this.terminated = false;
        if (rmiServer == null || connectionId == null) {
            throw new NullPointerException("Illegal null argument");
        }
        if (emptyMap == null) {
            emptyMap = Collections.emptyMap();
        }
        this.rmiServer = rmiServer;
        this.connectionId = connectionId;
        this.defaultClassLoader = defaultClassLoader;
        this.subjectDelegator = new SubjectDelegator();
        if ((this.subject = subject) == null) {
            this.acc = null;
            this.removeCallerContext = false;
        }
        else {
            this.removeCallerContext = SubjectDelegator.checkRemoveCallerContext(subject);
            if (this.removeCallerContext) {
                this.acc = JMXSubjectDomainCombiner.getDomainCombinerContext(subject);
            }
            else {
                this.acc = JMXSubjectDomainCombiner.getContext(subject);
            }
        }
        this.mbeanServer = rmiServer.getMBeanServer();
        this.classLoaderWithRepository = (ClassLoaderWithRepository)AccessController.doPrivileged(new PrivilegedAction<ClassLoaderWithRepository>() {
            
            @Override
            public ClassLoaderWithRepository run() {
                return new ClassLoaderWithRepository(repository, defaultClassLoader);
            }
        }, withPermissions(new Permission[] { new RuntimePermission("createClassLoader") }));
        this.defaultContextClassLoader = (ClassLoader)AccessController.doPrivileged(new PrivilegedAction<ClassLoader>() {
            @Override
            public ClassLoader run() {
                return new CombinedClassLoader(Thread.currentThread().getContextClassLoader(), defaultClassLoader);
            }
        });
        this.serverCommunicatorAdmin = new RMIConnectionImpl.RMIServerCommunicatorAdmin(EnvHelp.getServerConnectionTimeout(emptyMap));
        this.env = emptyMap;
        return;
    }
    
    private static AccessControlContext withPermissions(Permission... array) {
        Permissions permissions = new Permissions();
        for (int length = array.length, i = 0; i < length; ++i) {
            permissions.add(array[i]);
        }
        return new AccessControlContext(new ProtectionDomain[] { new ProtectionDomain(null, permissions) });
    }
    
    private synchronized ServerNotifForwarder getServerNotifFwd() {
        if (this.serverNotifForwarder == null) {
            this.serverNotifForwarder = new ServerNotifForwarder(this.mbeanServer, this.env, this.rmiServer.getNotifBuffer(), this.connectionId);
        }
        return this.serverNotifForwarder;
    }
    
    @Override
    public String getConnectionId() throws IOException {
        return this.connectionId;
    }
    
    @Override
    public void close() throws IOException {
        int debugOn = RMIConnectionImpl.logger.debugOn();
        String str = debugOn ? ("[" + this.toString() + "]") : null;
        synchronized (this) {
            if (this.terminated) {
                if (!(debugOn == 0)) {
                    RMIConnectionImpl.logger.debug("close", str + " already terminated.");
                }
                return;
            }
            else {
                if (!(debugOn == 0)) {
                    RMIConnectionImpl.logger.debug("close", str + " closing.");
                }
                this.terminated = true;
                if (this.serverCommunicatorAdmin != null) {
                    this.serverCommunicatorAdmin.terminate();
                }
                if (this.serverNotifForwarder != null) {
                    this.serverNotifForwarder.terminate();
                }
            }
        }
        this.rmiServer.clientClosed(this);
        if (!(debugOn == 0)) {
            RMIConnectionImpl.logger.debug("close", str + " closed.");
        }
    }
    
    @Override
    public void unreferenced() {
        RMIConnectionImpl.logger.debug("unreferenced", "called");
        try {
            this.close();
            RMIConnectionImpl.logger.debug("unreferenced", "done");
        }
        catch (IOException t) {
            RMIConnectionImpl.logger.fine("unreferenced", t);
        }
    }
    
    @Override
    public ObjectInstance createMBean(String str, ObjectName obj, Subject subject) throws ReflectionException, InstanceAlreadyExistsException, MBeanRegistrationException, MBeanException, NotCompliantMBeanException, IOException {
        try {
            Object[] array = { str, obj };
            if (RMIConnectionImpl.logger.debugOn()) {
                RMIConnectionImpl.logger.debug("createMBean(String,ObjectName)", "connectionId=" + this.connectionId + ", className=" + str + ", name=" + obj);
            }
            return (ObjectInstance)this.doPrivilegedOperation(3, array, subject);
        }
        catch (PrivilegedActionException ex) {
            Exception exception = extractException(ex);
            if ((exception instanceof ReflectionException)) {
                throw (ReflectionException)exception;
            }
            if ((exception instanceof InstanceAlreadyExistsException)) {
                throw (InstanceAlreadyExistsException)exception;
            }
            if ((exception instanceof MBeanRegistrationException)) {
                throw (MBeanRegistrationException)exception;
            }
            if ((exception instanceof MBeanException)) {
                throw (MBeanException)exception;
            }
            if ((exception instanceof NotCompliantMBeanException)) {
                throw (NotCompliantMBeanException)exception;
            }
            if ((exception instanceof IOException)) {
                throw (IOException)exception;
            }
            throw newIOException("Got unexpected server exception: " + exception, exception);
        }
    }
    
    @Override
    public ObjectInstance createMBean(String str, ObjectName obj, ObjectName obj2, Subject subject) throws ReflectionException, InstanceAlreadyExistsException, MBeanRegistrationException, MBeanException, NotCompliantMBeanException, InstanceNotFoundException, IOException {
        try {
            Object[] array = { str, obj, obj2 };
            if (RMIConnectionImpl.logger.debugOn()) {
                RMIConnectionImpl.logger.debug("createMBean(String,ObjectName,ObjectName)", "connectionId=" + this.connectionId + ", className=" + str + ", name=" + obj + ", loaderName=" + obj2);
            }
            return (ObjectInstance)this.doPrivilegedOperation(5, array, subject);
        }
        catch (PrivilegedActionException ex) {
            Exception exception = extractException(ex);
            if ((exception instanceof ReflectionException)) {
                throw (ReflectionException)exception;
            }
            if ((exception instanceof InstanceAlreadyExistsException)) {
                throw (InstanceAlreadyExistsException)exception;
            }
            if ((exception instanceof MBeanRegistrationException)) {
                throw (MBeanRegistrationException)exception;
            }
            if ((exception instanceof MBeanException)) {
                throw (MBeanException)exception;
            }
            if ((exception instanceof NotCompliantMBeanException)) {
                throw (NotCompliantMBeanException)exception;
            }
            if ((exception instanceof InstanceNotFoundException)) {
                throw (InstanceNotFoundException)exception;
            }
            if ((exception instanceof IOException)) {
                throw (IOException)exception;
            }
            throw newIOException("Got unexpected server exception: " + exception, exception);
        }
    }
    
    @Override
    public ObjectInstance createMBean(String str, ObjectName obj, MarshalledObject marshalledObject, String[] array, Subject subject) throws ReflectionException, InstanceAlreadyExistsException, MBeanRegistrationException, MBeanException, NotCompliantMBeanException, IOException {
        int debugOn = RMIConnectionImpl.logger.debugOn();
        if (!(debugOn == 0)) {
            RMIConnectionImpl.logger.debug("createMBean(String,ObjectName,Object[],String[])", "connectionId=" + this.connectionId + ", unwrapping parameters using classLoaderWithRepository.");
        }
        Object[] nullIsEmpty = nullIsEmpty((Object[])this.unwrap(marshalledObject, this.classLoaderWithRepository, Object[].class, subject));
        try {
            Object[] array2 = { str, obj, nullIsEmpty, nullIsEmpty(array) };
            if (!(debugOn == 0)) {
                RMIConnectionImpl.logger.debug("createMBean(String,ObjectName,Object[],String[])", "connectionId=" + this.connectionId + ", className=" + str + ", name=" + obj + ", signature=" + strings(array));
            }
            return (ObjectInstance)this.doPrivilegedOperation(4, array2, subject);
        }
        catch (PrivilegedActionException ex) {
            Exception exception = extractException(ex);
            if ((exception instanceof ReflectionException)) {
                throw (ReflectionException)exception;
            }
            if ((exception instanceof InstanceAlreadyExistsException)) {
                throw (InstanceAlreadyExistsException)exception;
            }
            if ((exception instanceof MBeanRegistrationException)) {
                throw (MBeanRegistrationException)exception;
            }
            if ((exception instanceof MBeanException)) {
                throw (MBeanException)exception;
            }
            if ((exception instanceof NotCompliantMBeanException)) {
                throw (NotCompliantMBeanException)exception;
            }
            if ((exception instanceof IOException)) {
                throw (IOException)exception;
            }
            throw newIOException("Got unexpected server exception: " + exception, exception);
        }
    }
    
    @Override
    public ObjectInstance createMBean(String str, ObjectName obj, ObjectName obj2, MarshalledObject marshalledObject, String[] array, Subject subject) throws ReflectionException, InstanceAlreadyExistsException, MBeanRegistrationException, MBeanException, NotCompliantMBeanException, InstanceNotFoundException, IOException {
        int debugOn = RMIConnectionImpl.logger.debugOn();
        if (!(debugOn == 0)) {
            RMIConnectionImpl.logger.debug("createMBean(String,ObjectName,ObjectName,Object[],String[])", "connectionId=" + this.connectionId + ", unwrapping params with MBean extended ClassLoader.");
        }
        Object[] nullIsEmpty = nullIsEmpty((Object[])this.unwrap(marshalledObject, this.getClassLoader(obj2), this.defaultClassLoader, Object[].class, subject));
        try {
            Object[] array2 = { str, obj, obj2, nullIsEmpty, nullIsEmpty(array) };
            if (!(debugOn == 0)) {
                RMIConnectionImpl.logger.debug("createMBean(String,ObjectName,ObjectName,Object[],String[])", "connectionId=" + this.connectionId + ", className=" + str + ", name=" + obj + ", loaderName=" + obj2 + ", signature=" + strings(array));
            }
            return (ObjectInstance)this.doPrivilegedOperation(6, array2, subject);
        }
        catch (PrivilegedActionException ex) {
            Exception exception = extractException(ex);
            if ((exception instanceof ReflectionException)) {
                throw (ReflectionException)exception;
            }
            if ((exception instanceof InstanceAlreadyExistsException)) {
                throw (InstanceAlreadyExistsException)exception;
            }
            if ((exception instanceof MBeanRegistrationException)) {
                throw (MBeanRegistrationException)exception;
            }
            if ((exception instanceof MBeanException)) {
                throw (MBeanException)exception;
            }
            if ((exception instanceof NotCompliantMBeanException)) {
                throw (NotCompliantMBeanException)exception;
            }
            if ((exception instanceof InstanceNotFoundException)) {
                throw (InstanceNotFoundException)exception;
            }
            if ((exception instanceof IOException)) {
                throw (IOException)exception;
            }
            throw newIOException("Got unexpected server exception: " + exception, exception);
        }
    }
    
    @Override
    public void unregisterMBean(ObjectName obj, Subject subject) throws InstanceNotFoundException, MBeanRegistrationException, IOException {
        try {
            Object[] array = { obj };
            if (RMIConnectionImpl.logger.debugOn()) {
                RMIConnectionImpl.logger.debug("unregisterMBean", "connectionId=" + this.connectionId + ", name=" + obj);
            }
            this.doPrivilegedOperation(24, array, subject);
        }
        catch (PrivilegedActionException ex) {
            Exception exception = extractException(ex);
            if ((exception instanceof InstanceNotFoundException)) {
                throw (InstanceNotFoundException)exception;
            }
            if ((exception instanceof MBeanRegistrationException)) {
                throw (MBeanRegistrationException)exception;
            }
            if ((exception instanceof IOException)) {
                throw (IOException)exception;
            }
            throw newIOException("Got unexpected server exception: " + exception, exception);
        }
    }
    
    @Override
    public ObjectInstance getObjectInstance(ObjectName obj, Subject subject) throws InstanceNotFoundException, IOException {
        checkNonNull("ObjectName", obj);
        try {
            Object[] array = { obj };
            if (RMIConnectionImpl.logger.debugOn()) {
                RMIConnectionImpl.logger.debug("getObjectInstance", "connectionId=" + this.connectionId + ", name=" + obj);
            }
            return (ObjectInstance)this.doPrivilegedOperation(13, array, subject);
        }
        catch (PrivilegedActionException ex) {
            Exception exception = extractException(ex);
            if ((exception instanceof InstanceNotFoundException)) {
                throw (InstanceNotFoundException)exception;
            }
            if ((exception instanceof IOException)) {
                throw (IOException)exception;
            }
            throw newIOException("Got unexpected server exception: " + exception, exception);
        }
    }
    
    @Override
    public Set<ObjectInstance> queryMBeans(ObjectName obj, MarshalledObject obj2, Subject subject) throws IOException {
        int debugOn = RMIConnectionImpl.logger.debugOn();
        if (!(debugOn == 0)) {
            RMIConnectionImpl.logger.debug("queryMBeans", "connectionId=" + this.connectionId + " unwrapping query with defaultClassLoader.");
        }
        QueryExp queryExp = (QueryExp)this.unwrap(obj2, this.defaultContextClassLoader, QueryExp.class, subject);
        try {
            Object[] array = { obj, queryExp };
            if (!(debugOn == 0)) {
                RMIConnectionImpl.logger.debug("queryMBeans", "connectionId=" + this.connectionId + ", name=" + obj + ", query=" + obj2);
            }
            return (Set)Util.cast(this.doPrivilegedOperation(17, array, subject));
        }
        catch (PrivilegedActionException ex) {
            Exception exception = extractException(ex);
            if ((exception instanceof IOException)) {
                throw (IOException)exception;
            }
            throw newIOException("Got unexpected server exception: " + exception, exception);
        }
    }
    
    @Override
    public Set<ObjectName> queryNames(ObjectName obj, MarshalledObject obj2, Subject subject) throws IOException {
        int debugOn = RMIConnectionImpl.logger.debugOn();
        if (!(debugOn == 0)) {
            RMIConnectionImpl.logger.debug("queryNames", "connectionId=" + this.connectionId + " unwrapping query with defaultClassLoader.");
        }
        QueryExp queryExp = (QueryExp)this.unwrap(obj2, this.defaultContextClassLoader, QueryExp.class, subject);
        try {
            Object[] array = { obj, queryExp };
            if (!(debugOn == 0)) {
                RMIConnectionImpl.logger.debug("queryNames", "connectionId=" + this.connectionId + ", name=" + obj + ", query=" + obj2);
            }
            return (Set)Util.cast(this.doPrivilegedOperation(18, array, subject));
        }
        catch (PrivilegedActionException ex) {
            Exception exception = extractException(ex);
            if ((exception instanceof IOException)) {
                throw (IOException)exception;
            }
            throw newIOException("Got unexpected server exception: " + exception, exception);
        }
    }
    
    @Override
    public boolean isRegistered(ObjectName objectName, Subject subject) throws IOException {
        try {
            return (boolean)this.doPrivilegedOperation(16, new Object[] { objectName }, subject);
        }
        catch (PrivilegedActionException ex) {
            Exception exception = extractException(ex);
            if ((exception instanceof IOException)) {
                throw (IOException)exception;
            }
            throw newIOException("Got unexpected server exception: " + exception, exception);
        }
    }
    
    @Override
    public Integer getMBeanCount(Subject subject) throws IOException {
        try {
            Object[] array = new Object[0];
            if (RMIConnectionImpl.logger.debugOn()) {
                RMIConnectionImpl.logger.debug("getMBeanCount", "connectionId=" + this.connectionId);
            }
            return (Integer)this.doPrivilegedOperation(11, array, subject);
        }
        catch (PrivilegedActionException ex) {
            Exception exception = extractException(ex);
            if ((exception instanceof IOException)) {
                throw (IOException)exception;
            }
            throw newIOException("Got unexpected server exception: " + exception, exception);
        }
    }
    
    @Override
    public Object getAttribute(ObjectName obj, String str, Subject subject) throws MBeanException, AttributeNotFoundException, InstanceNotFoundException, ReflectionException, IOException {
        try {
            Object[] array = { obj, str };
            if (RMIConnectionImpl.logger.debugOn()) {
                RMIConnectionImpl.logger.debug("getAttribute", "connectionId=" + this.connectionId + ", name=" + obj + ", attribute=" + str);
            }
            return this.doPrivilegedOperation(7, array, subject);
        }
        catch (PrivilegedActionException ex) {
            Exception exception = extractException(ex);
            if ((exception instanceof MBeanException)) {
                throw (MBeanException)exception;
            }
            if ((exception instanceof AttributeNotFoundException)) {
                throw (AttributeNotFoundException)exception;
            }
            if ((exception instanceof InstanceNotFoundException)) {
                throw (InstanceNotFoundException)exception;
            }
            if ((exception instanceof ReflectionException)) {
                throw (ReflectionException)exception;
            }
            if ((exception instanceof IOException)) {
                throw (IOException)exception;
            }
            throw newIOException("Got unexpected server exception: " + exception, exception);
        }
    }
    
    @Override
    public AttributeList getAttributes(ObjectName obj, String[] array, Subject subject) throws InstanceNotFoundException, ReflectionException, IOException {
        try {
            Object[] array2 = { obj, array };
            if (RMIConnectionImpl.logger.debugOn()) {
                RMIConnectionImpl.logger.debug("getAttributes", "connectionId=" + this.connectionId + ", name=" + obj + ", attributes=" + strings(array));
            }
            return (AttributeList)this.doPrivilegedOperation(8, array2, subject);
        }
        catch (PrivilegedActionException ex) {
            Exception exception = extractException(ex);
            if ((exception instanceof InstanceNotFoundException)) {
                throw (InstanceNotFoundException)exception;
            }
            if ((exception instanceof ReflectionException)) {
                throw (ReflectionException)exception;
            }
            if ((exception instanceof IOException)) {
                throw (IOException)exception;
            }
            throw newIOException("Got unexpected server exception: " + exception, exception);
        }
    }
    
    @Override
    public void setAttribute(ObjectName obj, MarshalledObject marshalledObject, Subject subject) throws InstanceNotFoundException, AttributeNotFoundException, InvalidAttributeValueException, MBeanException, ReflectionException, IOException {
        int debugOn = RMIConnectionImpl.logger.debugOn();
        if (!(debugOn == 0)) {
            RMIConnectionImpl.logger.debug("setAttribute", "connectionId=" + this.connectionId + " unwrapping attribute with MBean extended ClassLoader.");
        }
        Attribute attribute = (Attribute)this.unwrap(marshalledObject, this.getClassLoaderFor(obj), this.defaultClassLoader, Attribute.class, subject);
        try {
            Object[] array = { obj, attribute };
            if (!(debugOn == 0)) {
                RMIConnectionImpl.logger.debug("setAttribute", "connectionId=" + this.connectionId + ", name=" + obj + ", attribute name=" + attribute.getName());
            }
            this.doPrivilegedOperation(22, array, subject);
        }
        catch (PrivilegedActionException ex) {
            Exception exception = extractException(ex);
            if ((exception instanceof InstanceNotFoundException)) {
                throw (InstanceNotFoundException)exception;
            }
            if ((exception instanceof AttributeNotFoundException)) {
                throw (AttributeNotFoundException)exception;
            }
            if ((exception instanceof InvalidAttributeValueException)) {
                throw (InvalidAttributeValueException)exception;
            }
            if ((exception instanceof MBeanException)) {
                throw (MBeanException)exception;
            }
            if ((exception instanceof ReflectionException)) {
                throw (ReflectionException)exception;
            }
            if ((exception instanceof IOException)) {
                throw (IOException)exception;
            }
            throw newIOException("Got unexpected server exception: " + exception, exception);
        }
    }
    
    @Override
    public AttributeList setAttributes(ObjectName obj, MarshalledObject marshalledObject, Subject subject) throws InstanceNotFoundException, ReflectionException, IOException {
        int debugOn = RMIConnectionImpl.logger.debugOn();
        if (!(debugOn == 0)) {
            RMIConnectionImpl.logger.debug("setAttributes", "connectionId=" + this.connectionId + " unwrapping attributes with MBean extended ClassLoader.");
        }
        AttributeList list = (AttributeList)this.unwrap(marshalledObject, this.getClassLoaderFor(obj), this.defaultClassLoader, AttributeList.class, subject);
        try {
            Object[] array = { obj, list };
            if (!(debugOn == 0)) {
                RMIConnectionImpl.logger.debug("setAttributes", "connectionId=" + this.connectionId + ", name=" + obj + ", attribute names=" + RMIConnector.getAttributesNames(list));
            }
            return (AttributeList)this.doPrivilegedOperation(23, array, subject);
        }
        catch (PrivilegedActionException ex) {
            Exception exception = extractException(ex);
            if ((exception instanceof InstanceNotFoundException)) {
                throw (InstanceNotFoundException)exception;
            }
            if ((exception instanceof ReflectionException)) {
                throw (ReflectionException)exception;
            }
            if ((exception instanceof IOException)) {
                throw (IOException)exception;
            }
            throw newIOException("Got unexpected server exception: " + exception, exception);
        }
    }
    
    @Override
    public Object invoke(ObjectName obj, String str, MarshalledObject marshalledObject, String[] array, Subject subject) throws InstanceNotFoundException, MBeanException, ReflectionException, IOException {
        checkNonNull("ObjectName", obj);
        checkNonNull("Operation name", str);
        int debugOn = RMIConnectionImpl.logger.debugOn();
        if (!(debugOn == 0)) {
            RMIConnectionImpl.logger.debug("invoke", "connectionId=" + this.connectionId + " unwrapping params with MBean extended ClassLoader.");
        }
        Object[] nullIsEmpty = nullIsEmpty((Object[])this.unwrap(marshalledObject, this.getClassLoaderFor(obj), this.defaultClassLoader, Object[].class, subject));
        try {
            Object[] array2 = { obj, str, nullIsEmpty, nullIsEmpty(array) };
            if (!(debugOn == 0)) {
                RMIConnectionImpl.logger.debug("invoke", "connectionId=" + this.connectionId + ", name=" + obj + ", operationName=" + str + ", signature=" + strings(array));
            }
            return this.doPrivilegedOperation(14, array2, subject);
        }
        catch (PrivilegedActionException ex) {
            Exception exception = extractException(ex);
            if ((exception instanceof InstanceNotFoundException)) {
                throw (InstanceNotFoundException)exception;
            }
            if ((exception instanceof MBeanException)) {
                throw (MBeanException)exception;
            }
            if ((exception instanceof ReflectionException)) {
                throw (ReflectionException)exception;
            }
            if ((exception instanceof IOException)) {
                throw (IOException)exception;
            }
            throw newIOException("Got unexpected server exception: " + exception, exception);
        }
    }
    
    @Override
    public String getDefaultDomain(Subject subject) throws IOException {
        try {
            Object[] array = new Object[0];
            if (RMIConnectionImpl.logger.debugOn()) {
                RMIConnectionImpl.logger.debug("getDefaultDomain", "connectionId=" + this.connectionId);
            }
            return (String)this.doPrivilegedOperation(9, array, subject);
        }
        catch (PrivilegedActionException ex) {
            Exception exception = extractException(ex);
            if ((exception instanceof IOException)) {
                throw (IOException)exception;
            }
            throw newIOException("Got unexpected server exception: " + exception, exception);
        }
    }
    
    @Override
    public String[] getDomains(Subject subject) throws IOException {
        try {
            Object[] array = new Object[0];
            if (RMIConnectionImpl.logger.debugOn()) {
                RMIConnectionImpl.logger.debug("getDomains", "connectionId=" + this.connectionId);
            }
            return (String[])(String[])this.doPrivilegedOperation(10, array, subject);
        }
        catch (PrivilegedActionException ex) {
            Exception exception = extractException(ex);
            if ((exception instanceof IOException)) {
                throw (IOException)exception;
            }
            throw newIOException("Got unexpected server exception: " + exception, exception);
        }
    }
    
    @Override
    public MBeanInfo getMBeanInfo(ObjectName obj, Subject subject) throws InstanceNotFoundException, IntrospectionException, ReflectionException, IOException {
        checkNonNull("ObjectName", obj);
        try {
            Object[] array = { obj };
            if (RMIConnectionImpl.logger.debugOn()) {
                RMIConnectionImpl.logger.debug("getMBeanInfo", "connectionId=" + this.connectionId + ", name=" + obj);
            }
            return (MBeanInfo)this.doPrivilegedOperation(12, array, subject);
        }
        catch (PrivilegedActionException ex) {
            Exception exception = extractException(ex);
            if ((exception instanceof InstanceNotFoundException)) {
                throw (InstanceNotFoundException)exception;
            }
            if ((exception instanceof IntrospectionException)) {
                throw (IntrospectionException)exception;
            }
            if ((exception instanceof ReflectionException)) {
                throw (ReflectionException)exception;
            }
            if ((exception instanceof IOException)) {
                throw (IOException)exception;
            }
            throw newIOException("Got unexpected server exception: " + exception, exception);
        }
    }
    
    @Override
    public boolean isInstanceOf(ObjectName obj, String str, Subject subject) throws InstanceNotFoundException, IOException {
        checkNonNull("ObjectName", obj);
        try {
            Object[] array = { obj, str };
            if (RMIConnectionImpl.logger.debugOn()) {
                RMIConnectionImpl.logger.debug("isInstanceOf", "connectionId=" + this.connectionId + ", name=" + obj + ", className=" + str);
            }
            return (boolean)this.doPrivilegedOperation(15, array, subject);
        }
        catch (PrivilegedActionException ex) {
            Exception exception = extractException(ex);
            if ((exception instanceof InstanceNotFoundException)) {
                throw (InstanceNotFoundException)exception;
            }
            if ((exception instanceof IOException)) {
                throw (IOException)exception;
            }
            throw newIOException("Got unexpected server exception: " + exception, exception);
        }
    }
    
    @Override
    public Integer[] addNotificationListeners(ObjectName[] array, MarshalledObject[] array2, Subject[] array3) throws InstanceNotFoundException, IOException {
        if (array == null || array2 == null) {
            throw new IllegalArgumentException("Got null arguments.");
        }
        Subject[] array4 = (array3 != null) ? array3 : new Subject[array.length];
        if (array.length != array2.length || array2.length != array4.length) {
            throw new IllegalArgumentException("The value lengths of 3 parameters are not same.");
        }
        int i = 0;
        while (i < array.length) {
            if (array[i] == null) {
                throw new IllegalArgumentException("Null Object name.");
            }
            ++i;
        }
        int j = 0;
        NotificationFilter[] array5 = new NotificationFilter[array.length];
        Integer[] array6 = new Integer[array.length];
        int debugOn = RMIConnectionImpl.logger.debugOn();
        try {
            while (j < array.length) {
                ClassLoader classLoader = this.getClassLoaderFor(array[j]);
                if (!(debugOn == 0)) {
                    RMIConnectionImpl.logger.debug("addNotificationListener(ObjectName,NotificationFilter)", "connectionId=" + this.connectionId + " unwrapping filter with target extended ClassLoader.");
                }
                array5[j] = (NotificationFilter)this.unwrap(array2[j], classLoader, this.defaultClassLoader, NotificationFilter.class, array4[j]);
                if (!(debugOn == 0)) {
                    RMIConnectionImpl.logger.debug("addNotificationListener(ObjectName,NotificationFilter)", "connectionId=" + this.connectionId + ", name=" + array[j] + ", filter=" + array5[j]);
                }
                array6[j] = (Integer)this.doPrivilegedOperation(1, new Object[] { array[j], array5[j] }, array4[j]);
                ++j;
            }
            return array6;
        }
        catch (Exception exception) {
            for (int k = 0; k < j; ++k) {
                try {
                    this.getServerNotifFwd().removeNotificationListener(array[k], array6[k]);
                }
                catch (Exception ex) {}
            }
            if ((exception instanceof PrivilegedActionException)) {
                exception = extractException(exception);
            }
            if ((exception instanceof ClassCastException)) {
                throw (ClassCastException)exception;
            }
            if ((exception instanceof IOException)) {
                throw (IOException)exception;
            }
            if ((exception instanceof InstanceNotFoundException)) {
                throw (InstanceNotFoundException)exception;
            }
            if ((exception instanceof RuntimeException)) {
                throw (RuntimeException)exception;
            }
            throw newIOException("Got unexpected server exception: " + exception, exception);
        }
    }
    
    @Override
    public void addNotificationListener(ObjectName obj, ObjectName obj2, MarshalledObject marshalledObject, MarshalledObject marshalledObject2, Subject subject) throws InstanceNotFoundException, IOException {
        checkNonNull("Target MBean name", obj);
        checkNonNull("Listener MBean name", obj2);
        int debugOn = RMIConnectionImpl.logger.debugOn();
        ClassLoader classLoader = this.getClassLoaderFor(obj);
        if (!(debugOn == 0)) {
            RMIConnectionImpl.logger.debug("addNotificationListener(ObjectName,ObjectName,NotificationFilter,Object)", "connectionId=" + this.connectionId + " unwrapping filter with target extended ClassLoader.");
        }
        NotificationFilter obj3 = (NotificationFilter)this.unwrap(marshalledObject, classLoader, this.defaultClassLoader, NotificationFilter.class, subject);
        if (!(debugOn == 0)) {
            RMIConnectionImpl.logger.debug("addNotificationListener(ObjectName,ObjectName,NotificationFilter,Object)", "connectionId=" + this.connectionId + " unwrapping handback with target extended ClassLoader.");
        }
        Object unwrap = this.unwrap(marshalledObject2, classLoader, this.defaultClassLoader, Object.class, subject);
        try {
            Object[] array = { obj, obj2, obj3, unwrap };
            if (!(debugOn == 0)) {
                RMIConnectionImpl.logger.debug("addNotificationListener(ObjectName,ObjectName,NotificationFilter,Object)", "connectionId=" + this.connectionId + ", name=" + obj + ", listenerName=" + obj2 + ", filter=" + obj3 + ", handback=" + unwrap);
            }
            this.doPrivilegedOperation(2, array, subject);
        }
        catch (PrivilegedActionException ex) {
            Exception exception = extractException(ex);
            if ((exception instanceof InstanceNotFoundException)) {
                throw (InstanceNotFoundException)exception;
            }
            if ((exception instanceof IOException)) {
                throw (IOException)exception;
            }
            throw newIOException("Got unexpected server exception: " + exception, exception);
        }
    }
    
    @Override
    public void removeNotificationListeners(ObjectName obj, Integer[] array, Subject subject) throws InstanceNotFoundException, ListenerNotFoundException, IOException {
        if (obj == null || array == null) {
            throw new IllegalArgumentException("Illegal null parameter");
        }
        int i = 0;
        while (i < array.length) {
            if (array[i] == null) {
                throw new IllegalArgumentException("Null listener ID");
            }
            ++i;
        }
        try {
            Object[] array2 = { obj, array };
            if (RMIConnectionImpl.logger.debugOn()) {
                RMIConnectionImpl.logger.debug("removeNotificationListener(ObjectName,Integer[])", "connectionId=" + this.connectionId + ", name=" + obj + ", listenerIDs=" + objects(array));
            }
            this.doPrivilegedOperation(19, array2, subject);
        }
        catch (PrivilegedActionException ex) {
            Exception exception = extractException(ex);
            if ((exception instanceof InstanceNotFoundException)) {
                throw (InstanceNotFoundException)exception;
            }
            if ((exception instanceof ListenerNotFoundException)) {
                throw (ListenerNotFoundException)exception;
            }
            if ((exception instanceof IOException)) {
                throw (IOException)exception;
            }
            throw newIOException("Got unexpected server exception: " + exception, exception);
        }
        return;
    }
    
    @Override
    public void removeNotificationListener(ObjectName obj, ObjectName obj2, Subject subject) throws InstanceNotFoundException, ListenerNotFoundException, IOException {
        checkNonNull("Target MBean name", obj);
        checkNonNull("Listener MBean name", obj2);
        try {
            Object[] array = { obj, obj2 };
            if (RMIConnectionImpl.logger.debugOn()) {
                RMIConnectionImpl.logger.debug("removeNotificationListener(ObjectName,ObjectName)", "connectionId=" + this.connectionId + ", name=" + obj + ", listenerName=" + obj2);
            }
            this.doPrivilegedOperation(20, array, subject);
        }
        catch (PrivilegedActionException ex) {
            Exception exception = extractException(ex);
            if ((exception instanceof InstanceNotFoundException)) {
                throw (InstanceNotFoundException)exception;
            }
            if ((exception instanceof ListenerNotFoundException)) {
                throw (ListenerNotFoundException)exception;
            }
            if ((exception instanceof IOException)) {
                throw (IOException)exception;
            }
            throw newIOException("Got unexpected server exception: " + exception, exception);
        }
    }
    
    @Override
    public void removeNotificationListener(ObjectName obj, ObjectName obj2, MarshalledObject marshalledObject, MarshalledObject marshalledObject2, Subject subject) throws InstanceNotFoundException, ListenerNotFoundException, IOException {
        checkNonNull("Target MBean name", obj);
        checkNonNull("Listener MBean name", obj2);
        int debugOn = RMIConnectionImpl.logger.debugOn();
        ClassLoader classLoader = this.getClassLoaderFor(obj);
        if (!(debugOn == 0)) {
            RMIConnectionImpl.logger.debug("removeNotificationListener(ObjectName,ObjectName,NotificationFilter,Object)", "connectionId=" + this.connectionId + " unwrapping filter with target extended ClassLoader.");
        }
        NotificationFilter obj3 = (NotificationFilter)this.unwrap(marshalledObject, classLoader, this.defaultClassLoader, NotificationFilter.class, subject);
        if (!(debugOn == 0)) {
            RMIConnectionImpl.logger.debug("removeNotificationListener(ObjectName,ObjectName,NotificationFilter,Object)", "connectionId=" + this.connectionId + " unwrapping handback with target extended ClassLoader.");
        }
        Object unwrap = this.unwrap(marshalledObject2, classLoader, this.defaultClassLoader, Object.class, subject);
        try {
            Object[] array = { obj, obj2, obj3, unwrap };
            if (!(debugOn == 0)) {
                RMIConnectionImpl.logger.debug("removeNotificationListener(ObjectName,ObjectName,NotificationFilter,Object)", "connectionId=" + this.connectionId + ", name=" + obj + ", listenerName=" + obj2 + ", filter=" + obj3 + ", handback=" + unwrap);
            }
            this.doPrivilegedOperation(21, array, subject);
        }
        catch (PrivilegedActionException ex) {
            Exception exception = extractException(ex);
            if ((exception instanceof InstanceNotFoundException)) {
                throw (InstanceNotFoundException)exception;
            }
            if ((exception instanceof ListenerNotFoundException)) {
                throw (ListenerNotFoundException)exception;
            }
            if ((exception instanceof IOException)) {
                throw (IOException)exception;
            }
            throw newIOException("Got unexpected server exception: " + exception, exception);
        }
    }
    
    @Override
    public NotificationResult fetchNotifications(final long n, final int n2, final long lng) throws IOException {
        if (RMIConnectionImpl.logger.debugOn()) {
            RMIConnectionImpl.logger.debug("fetchNotifications", "connectionId=" + this.connectionId + ", timeout=" + lng);
        }
        if (n2 < 0 || lng < 0L) {
            throw new IllegalArgumentException("Illegal negative argument");
        }
        int reqIncoming = this.serverCommunicatorAdmin.reqIncoming();
        try {
            if (!(reqIncoming == 0)) {
                if (RMIConnectionImpl.logger.debugOn()) {
                    RMIConnectionImpl.logger.debug("fetchNotifications", "The notification server has been closed, returns null to force the client to stop fetching");
                }
                return null;
            }
            else {
                PrivilegedAction<NotificationResult> action = new PrivilegedAction<NotificationResult>() {
                    @Override
                    public NotificationResult run() {
                        return RMIConnectionImpl.access$200(RMIConnectionImpl.this).fetchNotifs(n, lng, n2);
                    }
                };
                if (this.acc == null) {
                    return (NotificationResult)action.run();
                }
                return (NotificationResult)AccessController.doPrivileged(action, this.acc);
            }
        }
        finally {
            this.serverCommunicatorAdmin.rspOutgoing();
        }
    }
    
    @Override
    public String toString() {
        return super.toString() + ": connectionId=" + this.connectionId;
    }
    
    private ClassLoader getClassLoader(final ObjectName objectName) throws InstanceNotFoundException {
        try {
            return (ClassLoader)AccessController.doPrivileged(new PrivilegedExceptionAction<ClassLoader>() {
                @Override
                public ClassLoader run() throws InstanceNotFoundException {
                    return RMIConnectionImpl.this.mbeanServer.getClassLoader(objectName);
                }
            }, withPermissions(new Permission[] { new MBeanPermission("*", "getClassLoader") }));
        }
        catch (PrivilegedActionException ex) {
            throw (InstanceNotFoundException)extractException(ex);
        }
    }
    
    private ClassLoader getClassLoaderFor(final ObjectName objectName) throws InstanceNotFoundException {
        try {
            return (ClassLoader)AccessController.doPrivileged(new PrivilegedExceptionAction<Object>() {
                @Override
                public Object run() throws InstanceNotFoundException {
                    return RMIConnectionImpl.this.mbeanServer.getClassLoaderFor(objectName);
                }
            }, withPermissions(new Permission[] { new MBeanPermission("*", "getClassLoaderFor") }));
        }
        catch (PrivilegedActionException ex) {
            throw (InstanceNotFoundException)extractException(ex);
        }
    }
    
    private Object doPrivilegedOperation(int n, Object[] array, Subject delegatedSubject) throws PrivilegedActionException, IOException {
        this.serverCommunicatorAdmin.reqIncoming();
        try {
            AccessControlContext context;
            if (delegatedSubject == null) {
                context = this.acc;
            }
            else {
                if (this.subject == null) {
                    throw new SecurityException("Subject delegation cannot be enabled unless an authenticated subject is put in place");
                }
                context = this.subjectDelegator.delegatedContext(this.acc, delegatedSubject, this.removeCallerContext);
            }
            RMIConnectionImpl.PrivilegedOperation action = new RMIConnectionImpl.PrivilegedOperation(n, array);
            if (context == null) {
                try {
                    return action.run();
                }
                catch (Exception exception) {
                    if ((exception instanceof RuntimeException)) {
                        throw (RuntimeException)exception;
                    }
                    throw new PrivilegedActionException(exception);
                }
            }
            else {
                return AccessController.doPrivileged(action, context);
            }
        }
        catch (Error err) {
            throw new JMXServerErrorException(err.toString(), err);
        }
        finally {
            this.serverCommunicatorAdmin.rspOutgoing();
        }
    }
    
    private Object doOperation(int n, Object[] array) throws Exception {
        switch (n) {
            case 3: {
                return this.mbeanServer.createMBean((String)array[0], (ObjectName)array[1]);
            }
            case 5: {
                return this.mbeanServer.createMBean((String)array[0], (ObjectName)array[1], (ObjectName)array[2]);
            }
            case 4: {
                return this.mbeanServer.createMBean((String)array[0], (ObjectName)array[1], (Object[])(Object[])array[2], (String[])(String[])array[3]);
            }
            case 6: {
                return this.mbeanServer.createMBean((String)array[0], (ObjectName)array[1], (ObjectName)array[2], (Object[])(Object[])array[3], (String[])(String[])array[4]);
            }
            case 7: {
                return this.mbeanServer.getAttribute((ObjectName)array[0], (String)array[1]);
            }
            case 8: {
                return this.mbeanServer.getAttributes((ObjectName)array[0], (String[])(String[])array[1]);
            }
            case 9: {
                return this.mbeanServer.getDefaultDomain();
            }
            case 10: {
                return this.mbeanServer.getDomains();
            }
            case 11: {
                return this.mbeanServer.getMBeanCount();
            }
            case 12: {
                return this.mbeanServer.getMBeanInfo((ObjectName)array[0]);
            }
            case 13: {
                return this.mbeanServer.getObjectInstance((ObjectName)array[0]);
            }
            case 14: {
                return this.mbeanServer.invoke((ObjectName)array[0], (String)array[1], (Object[])(Object[])array[2], (String[])(String[])array[3]);
            }
            case 15: {
                return this.mbeanServer.isInstanceOf((ObjectName)array[0], (String)array[1]) ? Boolean.TRUE : Boolean.FALSE;
            }
            case 16: {
                return this.mbeanServer.isRegistered((ObjectName)array[0]) ? Boolean.TRUE : Boolean.FALSE;
            }
            case 17: {
                return this.mbeanServer.queryMBeans((ObjectName)array[0], (QueryExp)array[1]);
            }
            case 18: {
                return this.mbeanServer.queryNames((ObjectName)array[0], (QueryExp)array[1]);
            }
            case 22: {
                this.mbeanServer.setAttribute((ObjectName)array[0], (Attribute)array[1]);
                return null;
            }
            case 23: {
                return this.mbeanServer.setAttributes((ObjectName)array[0], (AttributeList)array[1]);
            }
            case 24: {
                this.mbeanServer.unregisterMBean((ObjectName)array[0]);
                return null;
            }
            case 1: {
                return this.getServerNotifFwd().addNotificationListener((ObjectName)array[0], (NotificationFilter)array[1]);
            }
            case 2: {
                this.mbeanServer.addNotificationListener((ObjectName)array[0], (ObjectName)array[1], (NotificationFilter)array[2], array[3]);
                return null;
            }
            case 19: {
                this.getServerNotifFwd().removeNotificationListener((ObjectName)array[0], (Integer[])(Integer[])array[1]);
                return null;
            }
            case 20: {
                this.mbeanServer.removeNotificationListener((ObjectName)array[0], (ObjectName)array[1]);
                return null;
            }
            case 21: {
                this.mbeanServer.removeNotificationListener((ObjectName)array[0], (ObjectName)array[1], (NotificationFilter)array[2], array[3]);
                return null;
            }
            default: {
                throw new IllegalArgumentException("Invalid operation");
            }
        }
    }
    
    private <T> T unwrap(MarshalledObject<?> marshalledObject, ClassLoader classLoader, Class<T> clazz, Subject delegatedSubject) throws IOException {
        if (marshalledObject == null) {
            return null;
        }
        try {
            ClassLoader classLoader2 = (ClassLoader)AccessController.doPrivileged(new SetCcl(classLoader));
            try {
                AccessControlContext context;
                if (delegatedSubject == null) {
                    context = this.acc;
                }
                else {
                    if (this.subject == null) {
                        throw new SecurityException("Subject delegation cannot be enabled unless an authenticated subject is put in place");
                    }
                    context = this.subjectDelegator.delegatedContext(this.acc, delegatedSubject, this.removeCallerContext);
                }
                if (context != null) {
                    return AccessController.doPrivileged(()/*  */ -> p2.cast(p0.get()), context);
                }
                return clazz.cast(marshalledObject.get());
            }
            finally {
                AccessController.doPrivileged(new SetCcl(classLoader2));
            }
        }
        catch (PrivilegedActionException ex) {
            Exception exception = extractException(ex);
            if ((exception instanceof IOException)) {
                throw (IOException)exception;
            }
            if ((exception instanceof ClassNotFoundException)) {
                throw new UnmarshalException(exception.toString(), exception);
            }
            RMIConnectionImpl.logger.warning("unwrap", "Failed to unmarshall object: " + exception);
            RMIConnectionImpl.logger.debug("unwrap", exception);
        }
        catch (ClassNotFoundException ex2) {
            RMIConnectionImpl.logger.warning("unwrap", "Failed to unmarshall object: " + ex2);
            RMIConnectionImpl.logger.debug("unwrap", ex2);
            throw new UnmarshalException(ex2.toString(), ex2);
        }
        return null;
    }
    
    private <T> T unwrap(MarshalledObject<?> marshalledObject, final ClassLoader classLoader, final ClassLoader classLoader2, Class<T> clazz, Subject subject) throws IOException {
        if (marshalledObject == null) {
            return null;
        }
        try {
            return this.unwrap(marshalledObject, (ClassLoader)AccessController.doPrivileged(new PrivilegedExceptionAction<ClassLoader>() {
                @Override
                public ClassLoader run() throws Exception {
                    return new RMIConnectionImpl.CombinedClassLoader(Thread.currentThread().getContextClassLoader(), new OrderClassLoaders(classLoader, classLoader2));
                }
            }), clazz, subject);
        }
        catch (PrivilegedActionException ex) {
            Exception exception = extractException(ex);
            if ((exception instanceof IOException)) {
                throw (IOException)exception;
            }
            if ((exception instanceof ClassNotFoundException)) {
                throw new UnmarshalException(exception.toString(), exception);
            }
            RMIConnectionImpl.logger.warning("unwrap", "Failed to unmarshall object: " + exception);
            RMIConnectionImpl.logger.debug("unwrap", exception);
            return null;
        }
    }
    
    private static IOException newIOException(String message, Throwable cause) {
        return (IOException)EnvHelp.initCause(new IOException(message), cause);
    }
    
    private static Exception extractException(Exception exception) {
        while (exception instanceof PrivilegedActionException) {
            exception = ((PrivilegedActionException)exception).getException();
        }
        return exception;
    }
    
    private static Object[] nullIsEmpty(Object[] array) {
        return (array == null) ? RMIConnectionImpl.NO_OBJECTS : array;
    }
    
    private static String[] nullIsEmpty(String[] array) {
        return (array == null) ? RMIConnectionImpl.NO_STRINGS : array;
    }
    
    private static void checkNonNull(String str, Object o) {
        if (o == null) {
            throw new RuntimeOperationsException(new IllegalArgumentException(str + " must not be null"));
        }
        return;
    }
    
    private static String objects(Object[] a) {
        if (a == null) {
            return "null";
        }
        return Arrays.asList(a).toString();
    }
    
    private static String strings(String[] array) {
        return objects(array);
    }
    
    static ServerNotifForwarder access$200(RMIConnectionImpl rmiConnectionImpl) {
        return rmiConnectionImpl.getServerNotifFwd();
    }
    
    static Object access$300(RMIConnectionImpl rmiConnectionImpl, int n, Object[] array) throws Exception {
        return rmiConnectionImpl.doOperation(n, array);
    }
    
    static {
        NO_OBJECTS = new Object[0];
        NO_STRINGS = new String[0];
        logger = new ClassLogger("javax.management.remote.rmi", "RMIConnectionImpl");
    }
    
    private class PrivilegedOperation implements PrivilegedExceptionAction<Object>
    {
        private int operation;
        private Object[] params;
        
        public PrivilegedOperation(int operation, Object[] params) {
            this.operation = operation;
            this.params = params;
        }
        
        @Override
        public Object run() throws Exception {
            return RMIConnectionImpl.access$300(RMIConnectionImpl.this, this.operation, this.params);
        }
    }
    
    private class RMIServerCommunicatorAdmin extends ServerCommunicatorAdmin
    {
        public RMIServerCommunicatorAdmin(long timeout) {
            super(timeout);
        }
        
        @Override
        protected void doStop() {
            try {
                RMIConnectionImpl.this.close();
            }
            catch (IOException ex) {
                RMIConnectionImpl.logger.warning("RMIServerCommunicatorAdmin-doStop", "Failed to close: " + ex);
                RMIConnectionImpl.logger.debug("RMIServerCommunicatorAdmin-doStop", ex);
            }
        }
    }
    
    private static class SetCcl implements PrivilegedExceptionAction<ClassLoader>
    {
        private final ClassLoader classLoader;
        
        SetCcl(ClassLoader classLoader) {
            this.classLoader = classLoader;
        }
        
        @Override
        public ClassLoader run() {
            Thread currentThread = Thread.currentThread();
            ClassLoader contextClassLoader = currentThread.getContextClassLoader();
            currentThread.setContextClassLoader(this.classLoader);
            return contextClassLoader;
        }
    }
    
    private static final class CombinedClassLoader extends ClassLoader
    {
        final RMIConnectionImpl.CombinedClassLoader.ClassLoaderWrapper defaultCL;
        
        private CombinedClassLoader(ClassLoader parent, ClassLoader classLoader) {
            super(parent);
            this.defaultCL = new ClassLoaderWrapper(classLoader);
        }
        
        @Override
        protected Class<?> loadClass(String s, boolean resolve) throws ClassNotFoundException {
            ReflectUtil.checkPackageAccess(s);
            try {
                super.loadClass(s, resolve);
            }
            catch (Exception cause) {
                while (cause != null) {
                    if ((cause instanceof SecurityException)) {
                        throw (cause == cause) ? ((SecurityException)cause) : new SecurityException(cause.getMessage(), cause);
                    }
                    cause = cause.getCause();
                }
                return this.defaultCL.loadClass(s, resolve);
            }
            return this.defaultCL.loadClass(s, resolve);
        }
        
        private static final class ClassLoaderWrapper extends ClassLoader
        {
            ClassLoaderWrapper(ClassLoader parent) {
                super(parent);
            }
            
            @Override
            protected Class<?> loadClass(String name, boolean resolve) throws ClassNotFoundException {
                return super.loadClass(name, resolve);
            }
        }
    }
}
